#!/usr/bin/env python
#coding: utf-8
import pygame
from pygame.locals import *
import os
import math
import random
import sys

SCR_RECT = Rect(0,0,800,600)
TITLE,PLAY,GAMEOVER = (0,1,2)
position = [0,250]
pygame.joystick.init()
controller = pygame.joystick.Joystick(0)
controller.init()

class Main:
    def __init__(self):
        pygame.init()
        screen = pygame.display.set_mode(SCR_RECT.size)
        pygame.display.set_caption("おしめちゃん")
        #BGM
        pygame.mixer.music.load("n114.mp3")
        pygame.mixer.music.play(-1)
        # 素材のロード
        self.load_images()
        self.load_sounds()
        # ゲームの初期化
        self.init_game()
        clock = pygame.time.Clock()
        while True:
            clock.tick(60)
            screen.fill((0,0,0))
            self.update()
            self.draw(screen)
            pygame.display.update()
            self.key_handler()
    def init_game(self):
        """ゲームの初期化"""
        self.game_state = TITLE
        # スプライトグループの作成
        self.all = pygame.sprite.RenderUpdates()
        self.shots = pygame.sprite.Group()
        self.nomi = pygame.sprite.Group()
        self.explosion = pygame.sprite.Group()
        # デフォルトスプライトグループの登録
        Shimechan.containers = self.all
        Shot.containers = self.shots, self.all
        Boss.containers = self.all
        Nomi.containers = self.nomi, self.all
        ShimechanExplosion.containers = self.all
        # オブジェクトの作成
        self.battlefield = Battlefield()
        self.shimechan = Shimechan()
        self.boss = Boss()
    def update(self):
        if self.game_state == PLAY:
            self.battlefield.update()
            self.all.update()
            self.collide_detection()
    def draw(self,screen):
        if self.game_state == TITLE:
            screen.blit(self.title_image,(0,0))
        elif self.game_state == PLAY:
            screen.blit(self.battlefield.senzyou, (0,0), self.battlefield.offset())
            self.all.draw(screen)
        elif self.game_state == GAMEOVER:
            screen.blit(self.gameover_image,(0,0))
    def key_handler(self):
        for cue in pygame.event.get():
            if cue.type == QUIT:
                pygame.quit()
                sys.exit()
            if cue.type == KEYDOWN and cue.key == K_ESCAPE:
                pygame.quit()
                sys.exit()
            elif cue.type == KEYDOWN and cue.key == K_RETURN or cue.type == pygame.locals.JOYBUTTONDOWN and controller.get_button(0):
                if self.game_state == TITLE:
                    Shimechan.OPeffect.play()
                    self.init_game()
                    self.game_state = PLAY
                elif self.game_state == GAMEOVER:
                    self.game_state = TITLE
    def collide_detection(self):
        """衝突判定"""
        # 弾と敵機
        for pos in pygame.sprite.groupcollide(self.nomi, self.shots, 1, 1).keys():
            ShimechanExplosion(pos)
            Nomi.bomb_sound.play()
        # 弾とボス
        for pos in pygame.sprite.spritecollide(self.boss, self.shots, 1,  pygame.sprite.collide_circle):
            self.boss.Boss_HP -= 1
            ShimechanExplosion(pos)
            Nomi.bomb_sound.play()
            # ボスのHPが0になったらゲームオーバー画面へ
            if self.boss.Boss_HP == 0:
                self.game_state = GAMEOVER
        # 自機の爆発エフェクト
        for pos in pygame.sprite.spritecollide(self.shimechan, self.nomi, 1, pygame.sprite.collide_circle):
            ShimechanExplosion(pos)
            Shimechan.bomb_sound.play()
            self.game_state = GAMEOVER
    def load_images(self):
        """一枚絵から画像をロード"""
        sprite_sheet = SpriteSheet("oshimechan2.png")
        # 背景タイルの画像
        Battlefield.haikei = sprite_sheet.image_at((0,0,798,600))
        # 自機の画像
        Shimechan.image = sprite_sheet.image_at((1249,603,112,114), -1)
        # 自弾の画像
        Shot.image = sprite_sheet.image_at((1250,720,98,90), -1)
        # ボスの画像
        Boss.image = sprite_sheet.image_at((840,600,345,600), -1)
        # ノミの画像
        Nomi.image = sprite_sheet.image_at((1252,818,55,45), -1)
        # 着弾爆発エフェクトの画像
        ShimechanExplosion.images = sprite_sheet.images_at([(1249,876,52,52), (1315,876,52,52),
        (1381,876,52,52), (1447,876,52,52), (1249,943,52,52),
        (1315,943,52,52), (1381,943,52,52)], -1)
        # タイトル画像
        self.title_image = sprite_sheet.image_at((0,600,800,600))
        # ゲームオーバー画像
        self.gameover_image = sprite_sheet.image_at((800,0,800,600))
    def load_sounds(self):
        """サウンドのロード"""
        Shimechan.bomb_sound = load_sound("bom28_a.wav")
        Shimechan.shot_sound = load_sound("cat12.wav")
        Nomi.bomb_sound = load_sound("bom24.wav")
        Shimechan.OPeffect = load_sound("OPeffect.wav")

class SpriteSheet:
    """スプライトの一枚絵を管理するクラス"""
    def __init__(self, filename):
        self.sheet = load_image(filename)
    def image_at(self, rect, colorkey=None):
        """一枚絵からrectで指定した部分を切り取った画像を返す"""
        rect = Rect(rect)
        image = pygame.Surface(rect.size).convert()
        image.blit(self.sheet, (0, 0), rect)
        return image_colorkey(image, colorkey)
    def images_at(self, rects, colorkey=None):
        """一枚絵からrectsで指定した部分を切り取った画像リストを返す"""
        images = []
        for rect in rects:
            images.append(self.image_at(rect, colorkey))
        return images

class Battlefield:
    """戦場クラス"""
    speed = 1         # スクロール速度
    enemy_prob = 10   # 敵機の出現頻度
    def __init__(self):
        w = SCR_RECT.width
        h = SCR_RECT.height
        self.counter = 0
        self.rect = self.haikei.get_rect()
        self.tileside = self.haikei.get_width()
        self.senzyou = pygame.Surface((w*2, h)).convert()
        for x in range(int(w/self.tileside+1)):
            self.senzyou.blit(self.haikei, (x*self.tileside, 0))
    def offset(self):
        self.counter = (self.counter + self.speed) % SCR_RECT.width
        return (self.counter, 0, SCR_RECT.width, SCR_RECT.height)
    def update(self):
        # 0からenemy_probまでの乱数を出して0  # リロード時間
        # enemy_probが大きいと出にくくなる
        if not random.randrange(self.enemy_prob):
            Nomi()

class Shimechan(pygame.sprite.Sprite):
    """自機クラス"""
    reload_time = 20  # リロード時間
    def __init__(self):
        pygame.sprite.Sprite.__init__(self, self.containers)
        self.rect = self.image.get_rect()
        self.rect.left = SCR_RECT.left
        self.reload_timer = 0
        self.radius = 20
        self.vx0, self.vx1, self.vy0, self.vy1 = 0, 0, 0, 0
        pygame.key.set_repeat(1,1)
    def update(self):
        self.rect.center = position
        self.rect = self.rect.clamp(SCR_RECT)
        # 装填中
        if self.reload_timer > 0:
            self.reload_timer -= 1
        # 自機を操作する
        if controller.get_axis(0):
            position[0] += (round(controller.get_axis(0), 1) * 4)
        if controller.get_axis(1):
            position[1] += (round(controller.get_axis(1), 1) * 4)
        if controller.get_button(0):
            if self.reload_timer == 0:
                self.reload_timer = self.reload_time
                self.shot_sound.play()
                Shot((self.rect.center))

class Nomi(pygame.sprite.Sprite):
    """敵機クラス"""
    speed = -5        # 移動速度
    def __init__(self):
        pygame.sprite.Sprite.__init__(self, self.containers)
        # 出現位置はランダム
        self.rect = self.image.get_rect()
        self.rect.top = random.randrange(SCR_RECT.height - 100)
        self.rect.left = SCR_RECT.right
        self.yrange = random.randrange(-1, 1)
        self.radius = 20
    def update(self):
        # まっすぐ左へ移動
        self.rect.move_ip(self.speed, self.yrange)
        # 左端へ達したら消滅
        if self.rect.right < SCR_RECT.left:
            self.kill()

class Boss(pygame.sprite.Sprite):
    speed = 1
    up = True
    def __init__(self):
        pygame.sprite.Sprite.__init__(self, self.containers)
        self.rect = self.image.get_rect()
        self.rect.right = SCR_RECT.right
        self.Boss_HP = 70
        self.radius = 50
    def update(self):
        if self.rect.top == -50:
            self.up = False
        elif self.rect.bottom == 650:
            self.up = True
        if self.up == True:
            self.rect.move_ip(0, -self.speed)
        elif self.up == False:
            self.rect.move_ip(0, self.speed)

class ShimechanExplosion(pygame.sprite.Sprite):
    """爆発エフェクトクラス"""
    animcycle = 4
    def __init__(self, pos):
        pygame.sprite.Sprite.__init__(self, self.containers)
        self.image = self.images[0]
        self.frame = 0
        self.max_frame = len(self.images) * self.animcycle
        self.rect = self.image.get_rect()
        self.rect.center = pos.rect.center  # 着弾地の中心で爆発
    def update(self):
        self.image = self.images[self.frame//self.animcycle]
        self.frame += 1
        # アニメーションを最後まで再生したら消滅
        if self.frame == self.max_frame:
            self.kill()

class Shot(pygame.sprite.Sprite):
    """自機が発射する弾クラス"""
    speed = 9
    def __init__(self, pos):
        pygame.sprite.Sprite.__init__(self, self.containers)
        self.rect = self.image.get_rect()
        self.rect.center = pos
    def update(self):
        # 右に飛んでいき、画面外に出たら消滅
        self.rect.move_ip(self.speed, 0)
        if self.rect.left > SCR_RECT.width:
           self.kill()

def image_colorkey(image, colorkey):
    """画像にカラーキーを設定"""
    if colorkey is not None:
        if colorkey is -1:
            colorkey = image.get_at((0,0))
        image.set_colorkey(colorkey, RLEACCEL)
    return image

def load_image(filename, colorkey=None):
    """画像をロード"""
    filename = os.path.join(filename)
    image = pygame.image.load(filename).convert()
    return image_colorkey(image, colorkey)

def load_sound(filename):
    """サウンドをロード"""
    filename = os.path.join(filename)
    return pygame.mixer.Sound(filename)

if __name__ == '__main__':
    Main()
